﻿# -*- coding: utf-8 -*-

import bpy

#### =======================================================================
class OBJECT_OT_Grouping(bpy.types.Operator):
	'''Emptyでグループ化'''
	bl_idname = "object.grouping"
	bl_label  = "Emptyでグループ化"
	bl_options= {"REGISTER", "UNDO"}

	name: bpy.props.StringProperty(name="Name", description="Emptyに付ける名", default="group")
	type: bpy.props.EnumProperty(name="Type", default="PLAIN_AXES",
		items=(
			("PLAIN_AXES", "Plain Axes", "", "EMPTY_AXIS", 0),
			("ARROWS", "Arrows", "", "EMPTY_ARROWS", 1),
			("SINGLE_ARROW", "Single Arrow", "", "EMPTY_SINGLE_ARROW", 2),
			("CIRCLE", "Circle", "", "MESH_CIRCLE", 3),
			("CUBE", "Cube", "", "CUBE", 4),
			("SPHERE", "Sphere", "", "SPHERE", 5),
			("CONE", "Cone", "", "CONE", 6),
			("IMAGE", "Image", "", "FILE_IMAGE", 7),
		))
	keep_transform: bpy.props.BoolProperty(name="トランスフォームを維持", description="ペアレント前にトランスフォームを適用します", default=True)

	#### --------------------------------
	def execute(self, context):
		'''object.add()/empty_add()は仕様バグでradius/size 0が可能.
		名称重複時, name = nameでは既存objの名が変わる. .new()では新規objの名が変わる.
		'''
		"""
		selLst = context.selected_objects[:]	# 選択取得, 中身は破棄されるので実体を取る
		bpy.ops.object.empty_add(type= self.type, radius=0)	# collectionには自動リンク?
		ob = context.selected_objects[0]		# 元選択は破棄され, 作成結果が選択に入る.
		ob.name = self.name
		context.view_layer.objects.active = ob	# 親としてactive objに設定

		if(selLst):
			for o in selLst:					# 元選択の追加
				o.select_set(True)
			bpy.ops.object.parent_set(type="OBJECT", xmirror=False, keep_transform= self.keep_transform)
		"""

		ob = bpy.data.objects.new(self.name, None)	# None == Empty, 選択状態は変わらない
		ob.empty_display_size = 0.0					# プロパティの制限で0.0001にされる, setattr(,,0)でも不可
		ob.empty_display_type = self.type

		hasSelect = bool(context.selected_objects)

		# 選択内のactiveなobjectが属するcollectionの配下にする
		# 選択無しでもactiveは在るが、削除直後ではactiveも無い
		if(hasSelect):
			context.view_layer.objects.active.users_collection[0].objects.link(ob)
		else:
			context.view_layer.active_layer_collection.collection.objects.link(ob)
		ob.select_set(True)
		context.view_layer.objects.active = ob	# 親としてactive objに設定

		if(hasSelect):
			bpy.ops.object.parent_set(type="OBJECT", xmirror=False, keep_transform= self.keep_transform)

		return({"FINISHED"})
