﻿# -*- coding: utf-8 -*-

import bpy

#### =======================================================================
class OBJECT_OT_RemoveHierarchies(bpy.types.Operator):
	'''選択した階層下のオブジェクトを可視フラグ等を無視して削除する'''
	bl_idname = "object.remove_hierarchies"
	bl_label  = "オブジェクトを階層ごと削除"
	bl_options= {"REGISTER","UNDO"}

	#### --------------------------------
	def _FindRootObjects(self, ob):
		'''選択物のうちで上流の物だけを拾い出す'''
		selSet = set(ob)

		res = []
		APD = res.append
		# 選択とアクティブの重複から, setを利用する
		for o in selSet:
			p = o.parent
			while(p):
				if(p in selSet):
					break
				p = p.parent
			else:
				APD(o)
		return(res)

	#### --------------------------------
	def _GetChildren(self, ob):
		res = []
		for o in ob:
			res.extend(self.__GetChildren(o))
		return(res)

	def __GetChildren(self, o):
		r = [o]
		for c in o.children:
			r.extend(self.__GetChildren(c))
		return(r)

	#### --------------------------------
	def execute(self, context):
		'''選択無しアクティブ有りを認める.'''
		if((not context.selected_objects) and (not context.active_object)):
			return({"CANCELLED"})

		ob = self._GetChildren(self._FindRootObjects(context.selected_objects[:] + [context.active_object]))

		res = [ o.name for o in ob ]

		fn = bpy.data.objects
		for o in ob:
			fn.remove(o)

		if(res):
			self.report({"OPERATOR"}, ",".join(res))

		return({"FINISHED"})

#### =======================================================================
def AddMenu(self, context):
	'''VIEW3D_MT_object 追加用'''
	self.layout.separator()
	self.layout.operator(OBJECT_OT_RemoveHierarchies.bl_idname, text= OBJECT_OT_RemoveHierarchies.bl_label, icon="TRASH")
