﻿# -*- coding: utf-8 -*-

import bpy
import bl_ui.properties_paint_common as ppc

from bl_ui.space_view3d_toolbar import (
	VIEW3D_PT_tools_brush_color,
	VIEW3D_PT_tools_brush_swatches,
	VIEW3D_PT_tools_brush_clone,
	VIEW3D_PT_tools_brush_options,
	)
from bl_ui.space_image import (
	IMAGE_PT_paint_color,
	IMAGE_PT_paint_swatches,
	IMAGE_PT_paint_clone,
	IMAGE_PT_paint_options,
	)
if(bpy.app.version[1] == 80):
	from bl_ui.space_view3d_toolbar import VIEW3D_PT_tools_brush_gradient
	from bl_ui.space_image import IMAGE_PT_paint_gradient

#### =======================================================================
__all__ = ("UL_DrawFuncs", "FlagData", "UpdateProp", "FindBrushIndex", "SearchBrush",
	"VIEW3D_PT_PaintBrushSelector_tools_brush_color",
	"VIEW3D_PT_PaintBrushSelector_tools_brush_swatches",
	"VIEW3D_PT_PaintBrushSelector_tools_brush_clone",
	"VIEW3D_PT_PaintBrushSelector_tools_brush_options",
	"IMAGE_PT_PaintBrushSelector_paint_color",
	"IMAGE_PT_PaintBrushSelector_paint_swatches",
	"IMAGE_PT_PaintBrushSelector_paint_clone",
	"IMAGE_PT_PaintBrushSelector_paint_options",
	)
if(bpy.app.version[1] == 80):
	__all__ += (
		"VIEW3D_PT_PaintBrushSelector_tools_brush_gradient",
		"IMAGE_PT_PaintBrushSelector_paint_gradient",
		)

#### =======================================================================
class UL_DrawFuncs(object):
	def draw_item(self, context, layout, data, brush, icon, active_data, active_propName, index, filt_flg):
		if(filt_flg):
			layout.prop(brush, "name", text="", emboss=False, translate=False, icon_value=icon)
		else:
			layout.label(text="", translate=False)

	def draw_filter(self, context, layout):
		row = layout.row(align=True)
		row.prop(self, "filter_name", text="")
		row.prop(self, "use_filter_invert", icon_only=True, icon= "ZOOM_OUT" if(self.use_filter_invert) else "ZOOM_IN")
		row.prop(self, "use_filter_sort_alpha", icon_only=True)
		row.prop(self, "use_filter_sort_reverse", icon_only=True, icon= "SORT_DESC" if(self.use_filter_sort_reverse) else "SORT_ASC")

#### =======================================================================
class FlagData(object):
	__slots__ = ("isCutoffDep", "isDirty", "toolId")
	def __init__(self):
		self.isCutoffDep = False
		self.isDirty = False
		self.toolId = ""

	def Flush(self):
		self.isDirty = False
		self.isCutoffDep = False

	def SetCutoffDep(self):
		self.isCutoffDep = True

	def SetDirty(self, toolId):
		self.isDirty = True
		if(toolId):
			self.toolId = toolId

#### =======================================================================
def UpdateProp(scene, context, inst, setting, idx):
	'''GUIでリスト操作(idx変化問わず)→updateProp→brush更新→updateDep.'''
	if(setting and (not inst.isCutoffDep)):	# dep→prop→depの再入防止
		# context.tool_settings.sculpt/vertex_paint/weight_paint/image_paint/particle_edit
		selected = context.blend_data.brushes[idx]
		if(setting.brush != selected):	# 無変更時は変更処理させない
			setting.brush = selected
		inst.SetCutoffDep()		# depをスキップさせる
	return(None)

#### -----------------------------------------------------------------------
def FindBrushIndex(scene, brushName, key):
	i = bpy.data.brushes.find(brushName)
	if(scene.get(key) != i):
		scene[key] = i

#### -----------------------------------------------------------------------
def SearchBrush(scene, context, brush, key):
	if(brush):	# アクティブブラシが無くなる仕様バグへの対処として.brushのチェックが必要
		dep = context.evaluated_depsgraph_get()
		if((dep.mode == "VIEWPORT") and (dep.id_type_updated("BRUSH") or dep.id_type_updated("SCENE"))):
			FindBrushIndex(scene, brush.name, key)

#### =======================================================================
class __SubView(object):
	bl_space_type = "VIEW_3D"
	bl_region_type = "UI"
	bl_category = "Tool"
	bl_context = ".paint_common"
	bl_parent_id = "VIEW3D_PT_PaintBrushSelector"
	bl_options = {'DEFAULT_CLOSED'}

#### -----------------------------------------------------------------------
class VIEW3D_PT_PaintBrushSelector_tools_brush_color(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubView):
	bl_label = "Color Picker"
	poll = VIEW3D_PT_tools_brush_color.poll
	draw = VIEW3D_PT_tools_brush_color.draw

#### -----------------------------------------------------------------------
class VIEW3D_PT_PaintBrushSelector_tools_brush_swatches(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubView):
	bl_label = "Color Palette"
	poll = VIEW3D_PT_tools_brush_swatches.poll
	draw = VIEW3D_PT_tools_brush_swatches.draw

#### -----------------------------------------------------------------------
if(bpy.app.version[1] == 80):
	class VIEW3D_PT_PaintBrushSelector_tools_brush_gradient(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubView):
		bl_label = "Gradient"
		poll = VIEW3D_PT_tools_brush_gradient.poll
		draw_header = VIEW3D_PT_tools_brush_gradient.draw_header
		draw = VIEW3D_PT_tools_brush_gradient.draw

#### -----------------------------------------------------------------------
class VIEW3D_PT_PaintBrushSelector_tools_brush_clone(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubView):
	bl_label = "Clone from Paint Slot"
	poll = VIEW3D_PT_tools_brush_clone.poll
	draw_header = VIEW3D_PT_tools_brush_clone.draw_header
	draw = VIEW3D_PT_tools_brush_clone.draw

#### -----------------------------------------------------------------------
class VIEW3D_PT_PaintBrushSelector_tools_brush_options(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubView):
	bl_label = "Options"
	draw = VIEW3D_PT_tools_brush_options.draw
	# poll無し

#### =======================================================================
class __SubImg(object):
	bl_space_type = "IMAGE_EDITOR"
	bl_region_type = "UI"
	bl_category = "Tool"
	bl_context = ".paint_common_2d"
	bl_parent_id = "IMAGE_PT_PaintBrushSelector"
	bl_options = {'DEFAULT_CLOSED'}

#### -----------------------------------------------------------------------
class IMAGE_PT_PaintBrushSelector_paint_color(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubImg):
	bl_label = "Color Picker"
	poll = IMAGE_PT_paint_color.poll
	draw = IMAGE_PT_paint_color.draw

#### -----------------------------------------------------------------------
class IMAGE_PT_PaintBrushSelector_paint_swatches(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubImg):
	bl_label = "Color Palette"
	poll = IMAGE_PT_paint_swatches.poll
	draw = IMAGE_PT_paint_swatches.draw

#### -----------------------------------------------------------------------
if(bpy.app.version[1] == 80):
	class IMAGE_PT_PaintBrushSelector_paint_gradient(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubImg):
		bl_label = "Gradient"
		poll = IMAGE_PT_paint_gradient.poll
		draw_header = IMAGE_PT_paint_gradient.draw_header
		draw = IMAGE_PT_paint_gradient.draw

#### -----------------------------------------------------------------------
class IMAGE_PT_PaintBrushSelector_paint_clone(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubImg):
	bl_label = "Clone from Image/UV Map"
	poll = IMAGE_PT_paint_clone.poll
	draw = IMAGE_PT_paint_clone.draw

#### -----------------------------------------------------------------------
class IMAGE_PT_PaintBrushSelector_paint_options(bpy.types.Panel, ppc.UnifiedPaintPanel, __SubImg):
	bl_label = "Options"
	poll = IMAGE_PT_paint_options.poll
	draw = IMAGE_PT_paint_options.draw
