﻿# -*- coding: utf-8 -*-

import bpy
import bl_ui.properties_paint_common as ppc
from bl_ui.space_view3d_toolbar import VIEW3D_PT_tools_grease_pencil_brush
from bpy.app.handlers import persistent

from .BrushSelector_Sub import (UL_DrawFuncs, FlagData, UpdateProp, FindBrushIndex, SearchBrush,)

#### =======================================================================
_inst = None

_gToolNameDct = {}
for (k, v) in (("Draw","DRAW"),("Fill","FILL"),("Erase","ERASE")):
	_gToolNameDct["builtin_brush.{}".format(k)] = v

for (k, v) in (("cursor","cursor"),("cutter","cutter"),("line","DRAW"),("arc","DRAW"),("curve","DRAW"),("box","DRAW"),("circle","DRAW")):
	_gToolNameDct["builtin.{}".format(k)] = v

_gPrimitiveToolSet = {"builtin.line","builtin.arc","builtin.curve","builtin.box","builtin.circle"}
_gNonBrushToolSet = {"cursor","cutter"}

#### =======================================================================
@persistent
def _UpdateDep(scene):
	'''tool切替には反応しない. 実行時はcontext.space_dataはNone.
	GUIでpreview選択(選択変化問わず)→updateDep(SCENE: dirty, BRUSH: clean)→prop→Dep
	GUIでリスト操作(idx変化問わず)→updateProp→brush更新→updateDep(SCENE: dirty, BRUSH: clean)
	ブラシ追加()→updateDep(SCENE: dirty, BRUSH: dirty)
	GUIでの操作は同じ要素を選んでもSCENEがdirtyになる.
	(NG: dep()内で.brushを変更すると循環に入る)'''
	context = bpy.context
	if(context.mode != "PAINT_GPENCIL"):
		return

	if(_inst.isDirty):
		curToolId = context.workspace.tools.from_space_view3d_mode(context.mode).idname
		if(curToolId not in _gNonBrushToolSet):
			if(curToolId in _gPrimitiveToolSet):
				name = scene.tool_settings.gpencil_paint.tool_slots[0].brush.name
			else:
				name = scene.tool_settings.gpencil_paint.brush.name
			FindBrushIndex(scene, name, "GPencilListSelector_Index")
		_inst.Flush()
		return

	if(not _inst.isCutoffDep):	# prop→depの中断,dep→prop→depの再入防止.
		SearchBrush(scene, context, scene.tool_settings.gpencil_paint.brush, "GPencilListSelector_Index")
	_inst.Flush()

#### =======================================================================
def _UpdateProp_Gp(scene, context):
	'''GUIでリスト操作(idx変化問わず)→updateProp→brush更新→updateDep.'''
	return(UpdateProp(scene, context, _inst, scene.tool_settings.gpencil_paint, scene.GPencilListSelector_Index))

#### =======================================================================
class VIEW3D_UL_GPencilListSelector(bpy.types.UIList, UL_DrawFuncs):
	''' '''
	# updateDepを起動する為のpropとupdateFunc, スコープはclass内(非インスタンス)縛り？
	forceUpdate: bpy.props.BoolProperty(default=True, options={"HIDDEN","SKIP_SAVE"}, update= lambda scn, ctx: None)

	def filter_items(self, context, data, propName):
		'''flag True==0x40000000, False==0'''
		curToolId = context.workspace.tools.from_space_view3d_mode(context.mode).idname
		if(_inst.toolId != curToolId):
			_inst.SetDirty(curToolId)
			self.forceUpdate = True	# 値はなんでもいい

		#アクティブブラシが無くなる仕様バグへの対処として.brushのチェックが必要
		if(not context.scene.tool_settings.gpencil_paint.brush):
			return([], [])

		toolName = _gToolNameDct[curToolId]
		if(toolName in _gNonBrushToolSet):
			return([], [])

		filter_flags = [ (int(brush.use_paint_grease_pencil and (brush.gpencil_tool == toolName)) << 30) for brush in data.brushes.values() ]
		return(filter_flags, [])

#### =======================================================================
class VIEW3D_PT_GPencilListSelector(bpy.types.Panel):
	''' '''
	bl_space_type = "VIEW_3D"
	bl_region_type = "UI"
	bl_context = ".greasepencil_paint"
	bl_label = "Brush"
	bl_category = "Tool"

	#### --------------------------------
	poll = VIEW3D_PT_tools_grease_pencil_brush.poll

	#### --------------------------------
	def draw_header_preset(self, context):
		self.layout.alignment = "RIGHT"
		brush = context.scene.tool_settings.gpencil_paint.brush
		self.layout.label(text= "({})".format(brush.name) if(brush) else "()", translate=False)

	#### --------------------------------
	def draw(self, context):
		'''self==UnifiedPaintPanel, template_list.maxrowsは効かない?'''
		layout = self.layout
		layout.use_property_split = True

		brush = context.scene.tool_settings.gpencil_paint.brush
		isBrushTool = (_gToolNameDct[context.workspace.tools.from_space_view3d_mode(context.mode).idname] not in _gNonBrushToolSet)

		clm = layout.column()
		clm.use_property_split = False

		row = clm.row(align=True)
		row.template_list("VIEW3D_UL_GPencilListSelector", "", context.blend_data, "brushes", context.scene, "GPencilListSelector_Index", rows=1, columns=1)
		row.enabled = isBrushTool and bool(brush)
		# アクティブブラシが無くなる仕様バグへの対処.
		# None時にリスト選択してもプレビュ側のpyがbrush.gpencil_settingsでエラーを吐く.

		clm = row.column()
		clm.operator("brush.add_gpencil", text="", icon="DUPLICATE")

		if(isBrushTool and brush):
#			clm.prop(brush, "users", slider=True, icon_only=True)
			clm.prop(brush, "use_fake_user", text="", icon_only=True)
		else:
			clm.label(icon="FAKE_USER_ON")

		clm.operator("gpencil.brush_presets_create", icon="HELP", text="")

#		clm = layout.column()
#		clm.template_search(context.scene.tool_settings.gpencil_paint, "brush", context.blend_data, "brushes")
#		clm.template_ID(context.scene.tool_settings.gpencil_paint, "brush", new="brush.add_gpencil", live_icon=False)

		if((not isBrushTool) or (not brush)):
			return

		if(brush.gpencil_tool in {"DRAW", "FILL"}):
			gp_settings = brush.gpencil_settings

			row = layout.row(align=True)
			row_mat = row.row()
			if(gp_settings.use_material_pin):
				row_mat.template_ID(gp_settings, "material", live_icon=True)
			else:
				row_mat.template_ID(context.active_object, "active_material", live_icon=False)
				row_mat.enabled = False

			row.prop(gp_settings, "use_material_pin", text="")

		if(not self.is_popover):
			ppc.brush_basic_gpencil_paint_settings(layout, context, brush, compact=True)

#### =======================================================================
clss = (
	VIEW3D_UL_GPencilListSelector,
	VIEW3D_PT_GPencilListSelector,
)

#### =======================================================================
def register():
	global _inst
	_inst = FlagData()

	bpy.types.Scene.GPencilListSelector_Index = bpy.props.IntProperty(default=0, min=0, subtype="UNSIGNED", options={"HIDDEN","SKIP_SAVE"}, update=_UpdateProp_Gp)
	bpy.app.handlers.depsgraph_update_post.append(_UpdateDep)
	Fn = bpy.utils.register_class
	for cls in clss:
		Fn(cls)

#### -----------------------------------------------------------------------
def unregister():
	Fn = bpy.utils.unregister_class
	for cls in clss:
		Fn(cls)
	bpy.app.handlers.depsgraph_update_post.remove(_UpdateDep)
	del bpy.types.Scene.GPencilListSelector_Index

	global _inst
	_inst = None
